@@ -1,13 +1,17 @@
 package Tk::Mwm;
-require Tk;
+
+use vars qw($VERSION);
+$VERSION = '3.012'; # $Id: //depot/Tk8/Mwm/Mwm.pm#12 $
+
+use Tk qw($XS_VERSION);
 require DynaLoader;
 
 use vars qw($VERSION);
-$VERSION = '3.008'; # $Id: //depot/Tk8/Mwm/Mwm.pm#8$
+$VERSION = '3.012'; # $Id: //depot/Tk8/Mwm/Mwm.pm#12 $
 
 use base  qw(DynaLoader);
 
-bootstrap Tk::Mwm $Tk::VERSION;
+bootstrap Tk::Mwm;
 
 package Tk;
 use Tk::Submethods ( 'mwm' => [qw(decorations ismwmrunning protocol transientfor)] );
