@@ -6,9 +6,9 @@ package Tk::Tree;
 # Chris Dean <ctdean@cogit.com>
 
 use vars qw($VERSION);
-$VERSION = '3.013'; # $Id: //depot/Tk8/Tixish/Tree.pm#13$
+$VERSION = '3.019'; # $Id: //depot/Tk8/Tixish/Tree.pm#19 $
 
-use Tk;
+use Tk ();
 use Tk::Derived;
 use Tk::HList;
 use base  qw(Tk::Derived Tk::HList);
@@ -26,12 +26,9 @@ sub Populate
 
  $w->ConfigSpecs(
         -ignoreinvoke => ['PASSIVE',  'ignoreInvoke', 'IgnoreInvoke', 0],
-        -opencmd      => ['CALLBACK', 'openCmd',      'OpenCmd',
-                          sub { $w->OpenCmd( @_ ) } ],
-        -indicatorcmd => ['CALLBACK', 'indicatorCmd',      'IndicatorCmd',
-                          sub { $w->IndicatorCmd( @_ ) } ],
-        -closecmd     => ['CALLBACK', 'closeCmd',     'CloseCmd',
-                          sub { $w->CloseCmd( @_ ) } ],
+        -opencmd      => ['CALLBACK', 'openCmd',      'OpenCmd', 'OpenCmd' ],
+        -indicatorcmd => ['CALLBACK', 'indicatorCmd', 'IndicatorCmd', 'IndicatorCmd'],
+        -closecmd     => ['CALLBACK', 'closeCmd',     'CloseCmd', 'CloseCmd'],
         -indicator    => ['SELF', 'indicator', 'Indicator', 1],
         -indent       => ['SELF', 'indent', 'Indent', 20],
         -width        => ['SELF', 'width', 'Width', 20],
