@@ -9,7 +9,7 @@ use Carp;
 use File::Basename;
 
 use vars qw($VERSION);
-$VERSION = '3.045'; # $Id: //depot/Tk8/Tk/MMutil.pm#45 $
+$VERSION = '3.052'; # $Id: //depot/Tk8/Tk/MMutil.pm#52 $
 
 use Tk::MakeDepend;
 
@@ -20,7 +20,7 @@ use vars qw($IsWin32);
 $IsWin32 = ($^O eq 'MSWin32' || $Config{'ccflags'} =~ /-D_?WIN32_?/)
            unless defined $IsWin32;
 
-@MYEXPORT = qw(perldepend cflags const_config constants installbin c_o xs_o makefile manifypods);
+@MYEXPORT = qw(pasthru perldepend cflags const_config constants installbin c_o xs_o makefile manifypods);
 
 sub arch_prune
 {
@@ -200,6 +200,17 @@ sub upgrade_pic
  return $flags;
 }
 
+sub pasthru
+{
+ my $self = shift;
+ my $str = $self->MM::pasthru;
+ if ($str =~ s/^\s+INC=.*\n//m)
+  {
+   return "\n#Do NOT pasthru INC for Tk - it is computed by subdir\n".$str;
+  }
+ return $str;
+}
+
 sub perldepend
 {
  my $self = shift;
@@ -222,6 +233,19 @@ sub perldepend
    my @inc   = split(/\s+/,$self->{'INC'});
    my @def   = split(/\s+/,$self->{'DEFINE'});
    push(@def,qw(-DWIN32 -D__WIN32__)) if ($IsWin32);
+   if ($^O eq 'cygwin')
+    {
+     push(@def,qw(-D__CYGWIN__));
+     if ($win_arch eq 'MSWin32')
+      {
+       push(@def,qw(-D__WIN32__)) unless $self->{'DEFINE'} =~ /-D__WIN32__/;
+       push(@def,qw(-DWIN32)) if $self->{'NAME'} eq 'Tk::pTk';
+      }
+     elsif ($win_arch eq 'x')
+      {
+       push(@def,qw(-U_WIN32));
+      }
+    }
    foreach (@inc)
     {
      s/\$\(TKDIR\)/$tk/g;
@@ -248,6 +272,11 @@ sub const_config
    $self->{'LDFLAGS'} =~ s/-(debug|pdb:\w+)\s+//g;
    $self->{'LDDLFLAGS'} =~ s/-(debug|pdb:\w+)\s+//g;
   }
+ elsif ($^O =~ /(openbsd)/i)
+  {
+   # -Bforcearchive is bad news for Tk - we don't want all of libpTk.a in all .so-s.
+   $self->{'LDDLFLAGS'} =~ s/-Bforcearchive\s*//g;
+  }
  return $self->MM::const_config;
 }
 
@@ -301,7 +330,11 @@ sub manifypods
  my $self = shift;
  # Maybe always call UNIX version - we HTMLize them later
  local $_ = $self->MM::manifypods;
- if ($] >= 5.003)
+ if ($] >= 5.00565)
+  {
+   s/(POD2MAN_EXE.*pod2man)/$1 --center "perl\/Tk Documentation" --release "Tk\$(VERSION)"/;
+  }
+ elsif ($] >= 5.003)
   {
    s/(POD2MAN_EXE.*pod2man)/$1 -center "perl\/Tk Documentation" -release "Tk\$(VERSION)"/;
   }
@@ -478,6 +511,25 @@ sub TkExtMakefile
      my $libs = $att{'LIBS'}->[0];
      $att{'LIBS'}->[0] = "$extra $libs";
     }
+   if ($^O eq 'cygwin')
+    {
+     # NOTE: use gcc -shared instead of dllwrap (ld2),
+     # dllwrap tries to resolve all symbols, even those
+     # that are brought in from libraries like libpTk.a
+     push(@opt,'LD' => 'gcc -shared');
+     if ($win_arch eq 'MSWin32')
+      {
+       my $extra = "-lcomdlg32 -lgdi32";
+       my $libs = $att{'LIBS'}->[0];
+       $att{'LIBS'}->[0] = "$extra $libs";
+       $att{'DEFINE'} .= ' -D__WIN32__';
+       $att{'DEFINE'} .= ' -DWIN32' if($att{'NAME'} eq 'Tk::pTk');
+      }
+     elsif ($win_arch eq 'x')
+      {
+       $att{'DEFINE'} .= ' -U_WIN32';
+      }
+    }
    if (delete $att{'ptk_include'})
     {
      $i = "-I$ptk $i" unless ($ptk eq '.');
