@@ -456,7 +456,9 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
       }
      else
       {
-       sv_setsv(sv,MakeReference((SV *) av));
+       SV *ref = MakeReference((SV *) av);
+       sv_setsv(sv,ref);
+       SvREFCNT_dec(ref);
       }
      return (AV *) SvRV(sv);
     }
@@ -468,6 +470,8 @@ Tcl_ListObjAppendElement (Tcl_Interp *interp, Tcl_Obj *listPtr,
 			    Tcl_Obj *objPtr)
 {
  AV *av = ForceList(interp,listPtr);
+ if (!objPtr)
+  objPtr = &PL_sv_undef;
  if (av)
   {
    av_push(av, objPtr);
@@ -480,7 +484,12 @@ AV *
 MaybeForceList(Tcl_Interp *interp, Tcl_Obj *sv)
 {
  AV *av;
- if (SvIOK(sv) || SvNOK(sv))
+ int object = sv_isobject(sv);
+ if (!object && SvROK(sv) && SvTYPE(SvRV(sv)) == SVt_PVAV)
+  {
+   return (AV *) SvRV(sv);
+  }
+ else if (!object && (SvIOK(sv) || SvNOK(sv)))
   {
    av = newAV();
    av_store(av,0,SvREFCNT_inc(sv));
@@ -500,7 +509,8 @@ MaybeForceList(Tcl_Interp *interp, Tcl_Obj *sv)
    /* If there was more than one element set the SV */
    if (av && av_len(av) > 0)
     {
-     sv_setsv(sv,MakeReference((SV *) av));
+     /* AV is mortal - so we want newRV not MakeReference as we need extra REFCNT */
+     sv_setsv(sv,newRV((SV *) av));
     }
    return av;
   }
