@@ -14,7 +14,6 @@
 #include "imgInt.h"
 #include <string.h>
 #include <stdlib.h>
-#include <stdio.h>
 #if defined(__STDC__) || defined(HAS_STDARG)
 #include <stdarg.h>
 #else
@@ -51,12 +50,12 @@ extern int unlink _ANSI_ARGS_((CONST char *));
  * Prototypes for local procedures defined in this file:
  */
 
-static int ChnMatchTIFF _ANSI_ARGS_((Tcl_Channel chan, char *fileName,
-	Tcl_Obj *format, int *widthPtr, int *heightPtr));
-static int ObjMatchTIFF _ANSI_ARGS_((Tcl_Obj *dataObj,
+static int ChnMatchTIFF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
+	CONST char *fileName, Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int ObjMatchTIFF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
 	Tcl_Obj *format, int *widthPtr, int *heightPtr));
 static int ChnReadTIFF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
-	char *fileName, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+	CONST char *fileName, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
 static int ObjReadTIFF _ANSI_ARGS_((Tcl_Interp *interp,
 	Tcl_Obj *dataObj, Tcl_Obj *format,
@@ -69,7 +68,7 @@ static int StringWriteTIFF _ANSI_ARGS_((Tcl_Interp *interp,
 	Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtTIFF = {
-    "TIFF",					/* name */
+    "tiff",					/* name */
     (Tk_ImageFileMatchProc *) ChnMatchTIFF,	/* fileMatchProc */
     (Tk_ImageStringMatchProc *) ObjMatchTIFF,	/* stringMatchProc */
     (Tk_ImageFileReadProc *) ChnReadTIFF,	/* fileReadProc */
@@ -133,15 +132,15 @@ static char *symbols[] = {
     "TIFFSetErrorHandler",
     "TIFFSetWarningHandler",
     "TIFFClientOpen",
-    "TIFFRegisterCODEC",
+    "TIFFRegisterCODEC",	/* not in libtiff.def */
     "TIFFError",
-    "TIFFPredictorInit",
-    "_TIFFMergeFieldInfo",
-    "TIFFFlushData1",
-    "_TIFFNoPostDecode",
+    "TIFFPredictorInit",	/* not in libtiff.def */
+    "_TIFFMergeFieldInfo",	/* not in libtiff.def */
+    "TIFFFlushData1",		/* not in libtiff.def */
+    "_TIFFNoPostDecode",	/* not in libtiff.def */
     "TIFFTileRowSize",
     "TIFFScanlineSize",
-    "_TIFFsetByteArray",
+    "_TIFFsetByteArray",	/* not in libtiff.def */
     "TIFFVSetField",
     "TIFFSwabArrayOfShort",
     (char *) NULL
@@ -335,12 +334,12 @@ static int closeDummy _ANSI_ARGS_((thandle_t));
 static tsize_t writeDummy _ANSI_ARGS_((thandle_t, tdata_t, tsize_t));
 
 static tsize_t readMFile _ANSI_ARGS_((thandle_t, tdata_t, tsize_t));
-static tsize_t seekMFile _ANSI_ARGS_((thandle_t, toff_t, int));
+static toff_t  seekMFile _ANSI_ARGS_((thandle_t, toff_t, int));
 static toff_t  sizeMFile _ANSI_ARGS_((thandle_t));
 
 static tsize_t readString _ANSI_ARGS_((thandle_t, tdata_t, tsize_t));
 static tsize_t writeString _ANSI_ARGS_((thandle_t, tdata_t, tsize_t));
-static tsize_t seekString _ANSI_ARGS_((thandle_t, toff_t, int));
+static toff_t  seekString _ANSI_ARGS_((thandle_t, toff_t, int));
 static toff_t  sizeString _ANSI_ARGS_((thandle_t));
 
 static char *errorMessage = NULL;
@@ -473,13 +472,13 @@ readMFile(fd, data, size)
     return (tsize_t) ImgRead((MFile *) fd, (char *) data, (int) size) ;
 }
 
-static tsize_t
+static toff_t
 seekMFile(fd, off, whence)
     thandle_t fd;
     toff_t off;
     int whence;
 {
-    return (tsize_t) ImgSeek((MFile *) fd, (int) off, whence);
+    return Tcl_Seek((Tcl_Channel) ((MFile *) fd)->data, (int) off, whence);
 }
 
 static toff_t
@@ -487,7 +486,8 @@ sizeMFile(fd)
     thandle_t fd;
 {
     int fsize;
-    return (fsize = ImgSeek((MFile *) fd, 0, SEEK_END)) < 0 ? 0 : (toff_t) fsize;
+    return (fsize = Tcl_Seek((Tcl_Channel) ((MFile *) fd)->data,
+	    (int) 0, SEEK_END)) < 0 ? 0 : (toff_t) fsize;
 }
 
 /*
@@ -537,7 +537,7 @@ writeString(fd, data, size)
     return size;
 }
 
-static tsize_t
+static toff_t
 seekString(fd, off, whence)
     thandle_t fd;
     toff_t off;
@@ -590,7 +590,8 @@ sizeString(fd)
  */
 
 static int
-ObjMatchTIFF(data, format, widthPtr, heightPtr)
+ObjMatchTIFF(interp, data, format, widthPtr, heightPtr)
+    Tcl_Interp *interp;
     Tcl_Obj *data;		/* the object containing the image data */
     Tcl_Obj *format;		/* the image format string */
     int *widthPtr;		/* where to put the string width */
@@ -598,6 +599,8 @@ ObjMatchTIFF(data, format, widthPtr, heightPtr)
 {
     MFile handle;
 
+    ImgFixObjMatchProc(&interp, &data, &format, &widthPtr, &heightPtr);
+
     if (!ImgReadInit(data, '\111', &handle) &&
 	    !ImgReadInit(data, '\115', &handle)) {
 	return 0;
@@ -606,14 +609,17 @@ ObjMatchTIFF(data, format, widthPtr, heightPtr)
     return CommonMatchTIFF(&handle, widthPtr, heightPtr);
 }
 
-static int ChnMatchTIFF(chan, fileName, format, widthPtr, heightPtr)
+static int ChnMatchTIFF(interp, chan, fileName, format, widthPtr, heightPtr)
+    Tcl_Interp *interp;
     Tcl_Channel chan;
-    char *fileName;
+    CONST char *fileName;
     Tcl_Obj *format;
     int *widthPtr, *heightPtr;
 {
     MFile handle;
 
+    ImgFixChanMatchProc(&interp, &chan, &fileName, &format, &widthPtr, &heightPtr);
+
     handle.data = (char *) chan;
     handle.state = IMG_CHAN;
 
@@ -706,13 +712,10 @@ static int ObjReadTIFF(interp, data, format, imageHandle,
     } else {
 	Tcl_Channel outchan;
 	tmpnam(tempFileName);
-	outchan = Tcl_OpenFileChannel(interp, tempFileName, "w", 0644);
+	outchan = ImgOpenFileChannel(interp, tempFileName, 0644);
 	if (!outchan) {
 	    return TCL_ERROR;
 	}
-	if (Tcl_SetChannelOption(interp, outchan, "-translation", "binary") != TCL_OK) {
-	    return TCL_ERROR;
-	}
 
 	count = ImgRead(&handle, buffer, 1024);
 	while (count == 1024) {
@@ -752,7 +755,7 @@ static int ChnReadTIFF(interp, chan, fileName, format, imageHandle,
 	destX, destY, width, height, srcX, srcY)
     Tcl_Interp *interp;
     Tcl_Channel chan;
-    char *fileName;
+    CONST char *fileName;
     Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
@@ -779,13 +782,10 @@ static int ChnReadTIFF(interp, chan, fileName, format, imageHandle,
     } else {
 	Tcl_Channel outchan;
 	tmpnam(tempFileName);
-	outchan = Tcl_OpenFileChannel(interp, tempFileName, "w", 0644);
+	outchan = ImgOpenFileChannel(interp, tempFileName, 0644);
 	if (!outchan) {
 	    return TCL_ERROR;
 	}
-	if (Tcl_SetChannelOption(interp, outchan, "-translation", "binary") != TCL_OK) {
-	    return TCL_ERROR;
-	}
 
 	count = Tcl_Read(chan, buffer, 1024);
 	while (count == 1024) {
@@ -914,11 +914,14 @@ static int StringWriteTIFF(interp, dataPtr, format, blockPtr)
     char tempFileName[256];
     Tcl_DString dstring;
     char *mode;
+    Tcl_DString data;
 
     if (load_tiff_library(interp) != TCL_OK) {
 	return TCL_ERROR;
     }
 
+    ImgFixStringWriteProc(&data, &interp, &dataPtr, &format, &blockPtr);
+
     if (ParseWriteFormat(interp, format, &comp, &mode) != TCL_OK) {
     	return TCL_ERROR;
     }
@@ -951,13 +954,10 @@ static int StringWriteTIFF(interp, dataPtr, format, blockPtr)
     if (tempFileName[0]) {
 	Tcl_Channel inchan;
 	char buffer[1024];
-	inchan = Tcl_OpenFileChannel(interp, tempFileName, "w", 0644);
+	inchan = ImgOpenFileChannel(interp, tempFileName, 0644);
 	if (!inchan) {
 	    return TCL_ERROR;
 	}
-	if (Tcl_SetChannelOption(interp, inchan, "-translation", "binary") != TCL_OK) {
-	    return TCL_ERROR;
-	}
 	ImgWriteInit(dataPtr, &handle);
 
 	result = Tcl_Read(inchan, buffer, 1024);
@@ -977,6 +977,9 @@ static int StringWriteTIFF(interp, dataPtr, format, blockPtr)
 	Tcl_DStringFree(&dstring);
     }
     ImgPutc(IMG_DONE, &handle);
+    if ((result == TCL_OK) && (dataPtr == &data)) {
+	Tcl_DStringResult(interp, dataPtr);
+    }
     return result;
 }
 
@@ -991,7 +994,7 @@ static int ChnWriteTIFF(interp, filename, format, blockPtr)
     Tcl_DString nameBuffer; 
     char *fullname, *mode;
 
-    if ((fullname=Tcl_TranslateFileName(interp,filename,&nameBuffer))==NULL) {
+    if (!(fullname=Tcl_TranslateFileName(interp, filename, &nameBuffer))) {
 	return TCL_ERROR;
     }
 
@@ -1001,10 +1004,11 @@ static int ChnWriteTIFF(interp, filename, format, blockPtr)
     }
 
     if (ParseWriteFormat(interp, format, &comp, &mode) != TCL_OK) {
+	Tcl_DStringFree(&nameBuffer);
     	return TCL_ERROR;
     }
 
-    if (!(tif = tiff.Open(fullname,mode))) {
+    if (!(tif = tiff.Open(fullname, mode))) {
 	Tcl_AppendResult(interp, filename, ": ", Tcl_PosixError(interp),
 		(char *)NULL);
 	Tcl_DStringFree(&nameBuffer);
@@ -1061,10 +1065,12 @@ static int ParseWriteFormat(interp, format, comp, mode)
 	    *comp = COMPRESSION_DEFLATE;
 	} else if ((c == 'j') && (!strncmp(compression,"jpeg",length))) {
 	    *comp = COMPRESSION_JPEG;
-	} else if ((c == 'l') && (length>1) && (!strncmp(compression,"logluv",length))) {
+	} else if ((c == 'l') && (!strncmp(compression,"logluv",length))) {
 	    *comp = COMPRESSION_SGILOG;
+/* disabled, because of patented lzw-algorithm.
 	} else if ((c == 'l') && (length>1) && (!strncmp(compression,"lzw",length))) {
 	    *comp = COMPRESSION_LZW;
+*/
 	} else if ((c == 'p') && (length>1) && (!strncmp(compression,"packbits",length))) {
 	    *comp = COMPRESSION_PACKBITS;
 	} else if ((c == 'p') && (length>1) && (!strncmp(compression,"pixarlog",length))) {
