@@ -1,4 +1,4 @@
-/* 
+/*
  * tkCanvArc.c --
  *
  *	This file implements arc items for canvas widgets.
@@ -179,12 +179,12 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_COLOR, "-fill", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(ArcItem, fillColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-offset", (char *) NULL, (char *) NULL,
-	"0,0", Tk_Offset(ArcItem, tsoffset),
+	"0 0", Tk_Offset(ArcItem, tsoffset),
 	TK_CONFIG_DONT_SET_DEFAULT, &offsetOption},
     {TK_CONFIG_COLOR, "-outline", (char *) NULL, (char *) NULL,
 	"black", Tk_Offset(ArcItem, outline.color), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-outlineoffset", (char *) NULL, (char *) NULL,
-	"0,0", Tk_Offset(ArcItem, outline.tsoffset),
+	"0 0", Tk_Offset(ArcItem, outline.tsoffset),
 	TK_CONFIG_DONT_SET_DEFAULT, &offsetOption},
     {TK_CONFIG_BITMAP, "-outlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(ArcItem, outline.stipple),
@@ -331,7 +331,7 @@ CreateArc(interp, canvas, itemPtr, argc, argv)
     if (argc==1) {
 	i = 1;
     } else {
-	char *arg = Tcl_GetStringFromObj(args[1], NULL);
+	char *arg = Tcl_GetStringFromObj(objv[1], NULL);
 	if ((argc>1) && (arg[0] == '-')
 		&& (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    i = 1;
@@ -433,7 +433,7 @@ ArcCoords(interp, canvas, itemPtr, argc, argv)
 	Tcl_SetObjResult(interp, obj);
     } else if ((argc == 1)||(argc == 4)) {
 	if (argc==1) {
-	    if (Tcl_ListObjGetElements(interp, args[0], &argc, &args) != TCL_OK) {
+	    if (Tcl_ListObjGetElements(interp, objv[0], &argc, &objv) != TCL_OK) {
 		return TCL_ERROR;
 	    } else if (argc != 4) {
 		sprintf(c0,"%d",argc);
@@ -442,13 +442,13 @@ ArcCoords(interp, canvas, itemPtr, argc, argv)
 		return TCL_ERROR;
 	    }
 	}
-	if ((Tk_CanvasGetCoordFromObj(interp, canvas, args[0],
+	if ((Tk_CanvasGetCoordFromObj(interp, canvas, objv[0],
  		    &arcPtr->bbox[0]) != TCL_OK)
-		|| (Tk_CanvasGetCoordFromObj(interp, canvas, args[1],
+		|| (Tk_CanvasGetCoordFromObj(interp, canvas, objv[1],
 		    &arcPtr->bbox[1]) != TCL_OK)
-		|| (Tk_CanvasGetCoordFromObj(interp, canvas, args[2],
+		|| (Tk_CanvasGetCoordFromObj(interp, canvas, objv[2],
 			&arcPtr->bbox[2]) != TCL_OK)
-		|| (Tk_CanvasGetCoordFromObj(interp, canvas, args[3],
+		|| (Tk_CanvasGetCoordFromObj(interp, canvas, objv[3],
 			&arcPtr->bbox[3]) != TCL_OK)) {
 	    return TCL_ERROR;
 	}
@@ -1066,7 +1066,7 @@ ArcToPoint(canvas, itemPtr, pointPtr)
     ArcItem *arcPtr = (ArcItem *) itemPtr;
     double vertex[2], pointAngle, diff, dist, newDist;
     double poly[8], polyDist, width, t1, t2;
-    int filled, angleInRange;                         
+    int filled, angleInRange;
 
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
@@ -1758,7 +1758,7 @@ ComputeArcOutline(canvas,arcPtr)
 
 static int
 HorizLineToArc(x1, x2, y, rx, ry, start, extent)
-    double x1, x2;		/* X-coords of endpoints of line segment. 
+    double x1, x2;		/* X-coords of endpoints of line segment.
 				 * X1 must be <= x2. */
     double y;			/* Y-coordinate of line segment. */
     double rx, ry;		/* These x- and y-radii define an oval
@@ -1823,7 +1823,7 @@ HorizLineToArc(x1, x2, y, rx, ry, start, extent)
 static int
 VertLineToArc(x, y1, y2, rx, ry, start, extent)
     double x;			/* X-coordinate of line segment. */
-    double y1, y2;		/* Y-coords of endpoints of line segment. 
+    double y1, y2;		/* Y-coords of endpoints of line segment.
 				 * Y1 must be <= y2. */
     double rx, ry;		/* These x- and y-radii define an oval
 				 * centered at the origin. */
@@ -2199,7 +2199,7 @@ StylePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
 					 * information about how to reclaim
 					 * storage for return string. */
-{         
+{
     Arg result = NULL;
     register Style *stylePtr = (Style *) (widgRec + offset);
 
@@ -2209,5 +2209,5 @@ StylePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 	return LangStringArg("chord");
     } else {
 	return LangStringArg("pieslice");
-    }                                     
+    }
 }
