@@ -106,7 +106,7 @@ static void		ComputeBitmapBbox _ANSI_ARGS_((Tk_Canvas canvas,
 static int		ConfigureBitmap _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
 			    char **argv, int flags));
-static int		CreateBitmap _ANSI_ARGS_((Tcl_Interp *interp,
+static int		CreateTkBitmap _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
 			    int argc, char **argv));
 static void		DeleteBitmap _ANSI_ARGS_((Tk_Canvas canvas,
@@ -128,7 +128,7 @@ static void		TranslateBitmap _ANSI_ARGS_((Tk_Canvas canvas,
 Tk_ItemType tkBitmapType = {
     "bitmap",				/* name */
     sizeof(BitmapItem),			/* itemSize */
-    CreateBitmap,			/* createProc */
+    CreateTkBitmap,			/* createProc */
     configSpecs,			/* configSpecs */
     ConfigureBitmap,			/* configureProc */
     BitmapCoords,			/* coordProc */
@@ -173,7 +173,7 @@ Tk_ItemType tkBitmapType = {
  */
 
 static int
-CreateBitmap(interp, canvas, itemPtr, argc, argv)
+CreateTkBitmap(interp, canvas, itemPtr, argc, argv)
     Tcl_Interp *interp;			/* Interpreter for error reporting. */
     Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
