@@ -1,4 +1,4 @@
-/* 
+/*
  * tkTextTag.c --
  *
  *	This module implements the "tag" subcommand of the widget command
@@ -23,6 +23,28 @@
  * Information used for parsing tag configuration information:
  */
 
+static Tk_CustomOption stateOption = {
+    Tk_StateParseProc,
+    Tk_StatePrintProc, (ClientData) 2
+};
+
+/* The "-elide" is only provided for compatibility with TkMan 2.0,
+ * but might be removed in the future. The option "-state" provides
+ * the same functionality and is preferred.
+ */
+
+static int	ElideParseProc _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, Tk_Window tkwin,
+		    Arg value, char *widgRec, int offset));
+static Arg	ElidePrintProc _ANSI_ARGS_((ClientData clientData,
+		    Tk_Window tkwin, char *widgRec, int offset,
+		    Tcl_FreeProc **freeProcPtr));
+
+static Tk_CustomOption elideOption = {
+    ElideParseProc,
+    ElidePrintProc, (ClientData) 0
+};
+
 static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_BORDER, "-background", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, border), TK_CONFIG_NULL_OK},
@@ -31,6 +53,9 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_STRING, "-borderwidth", (char *) NULL, (char *) NULL,
 	"0", Tk_Offset(TkTextTag, bdString),
 		TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-elide", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(TkTextTag, state),
+	TK_CONFIG_NULL_OK, &elideOption},
     {TK_CONFIG_BITMAP, "-fgstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
@@ -58,17 +83,17 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	(char *) NULL, Tk_Offset(TkTextTag, spacing2String), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-spacing3", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, spacing3String), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(TkTextTag, state),
+	TK_CONFIG_NULL_OK, &stateOption},
     {TK_CONFIG_LANGARG, "-tabs", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, tabString), TK_CONFIG_NULL_OK},
     {TK_CONFIG_LANGARG, "-underline", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, underlineString),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_LANGARG, "-elide", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(TkTextTag, elideString),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_UID, "-wrap", (char *) NULL, (char *) NULL,
+    {TK_CONFIG_CUSTOM, "-wrap", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, wrapMode),
-	TK_CONFIG_NULL_OK},
+	TK_CONFIG_NULL_OK, &textWrapModeOption},
     {TK_CONFIG_LANGARG, "-data", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, userData),
 	TK_CONFIG_NULL_OK},
@@ -158,7 +183,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		index2 = index1;
 		TkTextIndexForwChars(&index2, 1, &index2);
 	    }
-    
+
 	    if (tagPtr->affectsDisplay) {
 		TkTextRedrawTag(textPtr, &index1, &index2, tagPtr, !addTag);
 	    } else {
@@ -166,17 +191,17 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		 * Still need to trigger enter/leave events on tags that
 		 * have changed.
 		 */
-    
+
 		TkTextEventuallyRepick(textPtr);
 	    }
 	    TkBTreeTag(&index1, &index2, tagPtr, addTag);
-    
+
 	    /*
 	     * If the tag is "sel" then grab the selection if we're supposed
 	     * to export it and don't already have it.  Also, invalidate
 	     * partially-completed selection retrievals.
 	     */
-    
+
 	    if (tagPtr == textPtr->selTagPtr) {
 		if (addTag && textPtr->exportSelection
 			&& !(textPtr->flags & GOT_SELECTION)) {
@@ -218,7 +243,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		append = 1;
 	    }
 	    mask = Tk_CreateBinding(interp, textPtr->bindingTable,
-		    (ClientData) tagPtr, argv[4], args[5], append);
+		    (ClientData) tagPtr, argv[4], objv[5], append);
 	    if (mask == 0) {
 		return TCL_ERROR;
 	    }
@@ -236,15 +261,14 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		return TCL_ERROR;
 	    }
 	} else if (argc == 5) {
-	    Arg command;
-    
+	    Tcl_Obj *command;
+
 	    command = Tk_GetBinding(interp, textPtr->bindingTable,
 		    (ClientData) tagPtr, argv[4]);
 	    if (command == NULL) {
 		return TCL_ERROR;
 	    }
-	    Tcl_ArgResult(interp,command);
-	    Tcl_DecrRefCount(command);
+	    Tcl_SetObjResult(interp,command);
 	} else {
 	    Tk_GetAllBindings(interp, textPtr->bindingTable,
 		    (ClientData) tagPtr);
@@ -391,15 +415,6 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    return TCL_ERROR;
 		}
 	    }
-	    if ((tagPtr->wrapMode != NULL)
-		    && (tagPtr->wrapMode != tkTextCharUid)
-		    && (tagPtr->wrapMode != tkTextNoneUid)
-		    && (tagPtr->wrapMode != tkTextWordUid)) {
-		Tcl_AppendResult(interp, "bad wrap mode \"", tagPtr->wrapMode,
-			"\": must be char, none, or word", (char *) NULL);
-		tagPtr->wrapMode = NULL;
-		return TCL_ERROR;
-	    }
 
 	    /*
 	     * If the "sel" tag was changed, be sure to mirror information
@@ -432,8 +447,8 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    || (tagPtr->spacing3String != NULL)
 		    || (tagPtr->tabString != NULL)
 		    || (tagPtr->underlineString != NULL)
-		    || (tagPtr->elideString != NULL)
-		    || (tagPtr->wrapMode != NULL)) {
+		    || (tagPtr->state != TK_STATE_NULL)
+		    || (tagPtr->wrapMode != TEXT_WRAPMODE_NULL)) {
 		tagPtr->affectsDisplay = 1;
 	    }
 	    TkTextRedrawTag(textPtr, (TkTextIndex *) NULL,
@@ -596,7 +611,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 	     * skip to the end of this tagged range.
 	     */
 
-	    for (segPtr = index1.linePtr->segPtr, offset = index1.charIndex; 
+	    for (segPtr = index1.linePtr->segPtr, offset = index1.charIndex;
 		    offset >= 0;
 		    offset -= segPtr->size, segPtr = segPtr->nextPtr) {
 		if ((offset == 0) && (segPtr->typePtr == &tkTextToggleOnType)
@@ -826,9 +841,8 @@ TkTextCreateTag(textPtr, tagName)
     tagPtr->tabArrayPtr = NULL;
     tagPtr->underlineString = NULL;
     tagPtr->underline = 0;
-    tagPtr->elideString = NULL;
-    tagPtr->elide = 0;
-    tagPtr->wrapMode = NULL;
+    tagPtr->state = TK_STATE_NULL;
+    tagPtr->wrapMode = TEXT_WRAPMODE_NULL;
     tagPtr->userData = NULL;
     tagPtr->affectsDisplay = 0;
     textPtr->numTags++;
@@ -1400,3 +1414,90 @@ TkTextPickCurrent(textPtr, eventPtr)
 	ckfree((char *) copyArrayPtr);
     }
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * ElideParseProc --
+ *
+ *	This procedure is invoked during option processing to handle
+ *	the "-elide" option. This option is an obsolete equivalent
+ *	of the "-state" option. It is only added for compatibility
+ *	with TkMan 2.0, but will be removed in the future.
+ *
+ * Results:
+ *	A standard Tcl return value.
+ *
+ * Side effects:
+ *	The state for a given item gets replaced by the state
+ *	indicated in the value argument.
+ *
+ *--------------------------------------------------------------
+ */
+
+static int
+ElideParseProc(clientData, interp, tkwin, value, widgRec, offset)
+    ClientData clientData;		/* some flags.*/
+    Tcl_Interp *interp;			/* Used for reporting errors. */
+    Tk_Window tkwin;			/* Window containing canvas widget. */
+    Arg value;				/* Value of option. */
+    char *widgRec;			/* Pointer to record for item. */
+    int offset;				/* Offset into item. */
+{
+    int b;
+    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
+    char *svalue = LangString(value);
+
+    if(svalue == NULL || *svalue == 0) {
+	*statePtr = TK_STATE_NULL;
+	return TCL_OK;
+    }
+    if (Tcl_GetBoolean(interp, value, &b) != TCL_OK) {
+	*statePtr = TK_STATE_NULL;
+	return TCL_ERROR;
+    }
+    *statePtr = b?TK_STATE_HIDDEN:TK_STATE_NORMAL;
+    return TCL_OK;
+}
+/*
+ *--------------------------------------------------------------
+ *
+ * ElidePrintProc --
+ *
+ *	This procedure is invoked by the Tk configuration code
+ *	to produce a printable string for the "-elide"
+ *	configuration option.
+ *
+ * Results:
+ *	The return value is a string describing the state for
+ *	the item referred to by "widgRec".  In addition, *freeProcPtr
+ *	is filled in with the address of a procedure to call to free
+ *	the result string when it's no longer needed (or NULL to
+ *	indicate that the string doesn't need to be freed).
+ *
+ * Side effects:
+ *	None.
+ *
+ *--------------------------------------------------------------
+ */
+
+static Arg
+ElidePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+    ClientData clientData;		/* Ignored. */
+    Tk_Window tkwin;			/* Window containing text widget. */
+    char *widgRec;			/* Pointer to record for item. */
+    int offset;				/* Offset into item. */
+    Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
+					 * information about how to reclaim
+					 * storage for return string. */
+{
+    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
+
+    if (*statePtr==TK_STATE_HIDDEN) {
+	return Tcl_NewIntObj(1);
+    } else if (*statePtr==TK_STATE_NULL) {
+	return LangStringArg("");
+    } else {
+	return Tcl_NewIntObj(0);
+    }
+}
