@@ -13,7 +13,7 @@
 
 #include "pTk/tkPort.h"
 
-#ifdef WIN32
+#if defined(WIN32) || (defined(__WIN32__) && defined(__CYGWIN__))
 
 #include "pTk/tkWinInt.h"
 
